/*==========================================================================
	Datei:	IO_SAMA5D3x_h

	On-Chip Module Register und Harewareresourcen des Atmel SAMA5D3x
	Nicht alle Register definiert!
	(FEHLER NICHT AUSGESCLOSSEN !)

	Begonnen:			06.12.12
	Letzte nderung:	Heute
	Autor:				A. Heinze\HAMEG Chemnitz
===========================================================================*/
#ifndef IO_SAMA5D3x_h
#define IO_SAMA5D3x_h

#ifndef sama5d3x
#define sama5d3x
#endif

#include <AT_SAMA5D3x/SAMA5D3X.h>

//=- Fehlerliste
enum cpu_error_list
{
	CPU_ERROR_NONE,
	CPU_ERROR_WRONG_PARAMETER,
	CPU_ERROR_WRONG_IRQ_INDEX,
	CPU_ERROR_WRONG_PRIORITY,
	CPU_ERROR_WRONG_EXTIRQ_NO,
	CPU_ERROR_IRQ_PRIORITY_NO_WRITEABLE,
	CPU_ERROR_IRQ_VECTOR_RESERVED,
	CPU_ERROR_IRQ_TABLE_READ_ONLY,
	CPU_ERROR_EXT_IRQ_NUMBER_WRONG,
	CPU_ERROR_EXT_IRQ_LEVEL_WRONG,
	CPU_ERROR_EXT_IRQ_MAX_COUNT_PER_PIN,
};


#define SET_32BIT_REGISTER(address, value)		((address) |=  ((unsigned int)  (value)))
#define SET_16BIT_REGISTER(address, value)		((address) |=  ((unsigned short)(value)))
#define SET_8BIT_REGISTER(address, value)		((address) |=  ((unsigned char) (value)))
#define CLEAR_32BIT_REGISTER(address, value)	((address) &= ~((unsigned int)  (value)))
#define CLEAR_16BIT_REGISTER(address, value)	((address) &= ~((unsigned short)(value)))
#define CLEAR_8BIT_REGISTER(address, value)		((address) &= ~((unsigned char) (value)))
#define WRITE_32BIT_REGISTER(address, value)	((address) =   ((unsigned int)  (value)))
#define WRITE_16BIT_REGISTER(address, value)	((address) =   ((unsigned short)(value)))
#define WRITE_8BIT_REGISTER(address, value)		((address) =   ((unsigned char) (value)))
#define READ_REGISTER(address)					(address)


#define CPU_REGISTER_WIDTH		32					//Breite der Universalregister in Bit
#define CPU_MEM_CS_BITS			0xF0000000			//Maske fr CS Adressen (fr CS5 und CS6 0x1e000000)
#define CPU_CORE_TYPE_STR		"SAMA5D3x"
#define CPU_VECTOR_NUMBERS		128
#define CPU_VECTOR_TABLE_SIZE	(CPU_REGISTER_WIDTH / 8 * CPU_VECTOR_NUMBERS)	//Gre der Vektortabelle in Byte


#define CPU_16BIT_LENGTH				2
#define CPU_16BIT_ALIGN_MASK 			(CPU_16BIT_LENGTH - 1)
#define CPU_32BIT_LENGTH				4
#define CPU_32BIT_ALIGN_MASK 			(CPU_32BIT_LENGTH - 1)
#define CPU_CACHE_LINE_LENGTH			32
#define	CPU_CCL_ALIGN_MASK				(CPU_CACHE_LINE_LENGTH - 1)

//#define CPU_ALIGN_TO_16BIT(n)			(((n) & CPU_16BIT_ALIGN_MASK) != 0 ? ((n) & ~CPU_16BIT_ALIGN_MASK) + CPU_16BIT_LENGTH : (n))
//#define CPU_ALIGN_TO_32BIT(n)			(((n) & CPU_32BIT_ALIGN_MASK) != 0 ? ((n) & ~CPU_32BIT_ALIGN_MASK) + CPU_32BIT_LENGTH : (n))
#define CPU_ALIGN_TO_16BIT(n)			(((n) + CPU_16BIT_ALIGN_MASK) & ~CPU_16BIT_ALIGN_MASK)
#define CPU_ALIGN_TO_32BIT(n)			(((n) + CPU_32BIT_ALIGN_MASK) & ~CPU_32BIT_ALIGN_MASK)
#define CPU_ALIGN_TO_CACHELINE(n)		(((n) + CPU_CCL_ALIGN_MASK)   & ~CPU_CCL_ALIGN_MASK)

#define ADC_BIT_RESOLUTION				12					// Auflsung des integrierten AD-Wandlers

#ifdef SYS_INTERRUPT						//Schalter ob spezielle Interruptfunktionen mit RTE abgeschlossen werden
	#undef SYS_INTERRUPT
#endif

// SAMA5D31 hat TC1 NICHT!
#define CPU_TC1_AVAILABLE				FALSE


//==== CPU Takt Geschichten =============================================================
#define CPU_CPU_CORE_CLK_HZ			(((HW_MC_MAIN_CPU_CRYSTAL_HZ * (HW_MC_CLK_PLLMULT + 1)) / HW_MC_CLK_DIVA) / HW_MC_CLK_PLLDIV2)
#define CPU_CPU_MCLK_CLK_HZ			(CPU_CPU_CLK_HZ / HW_MC_CLK_MDIV)
#define CPU_CPU_CLK_HZ				CPU_CPU_CORE_CLK_HZ
#define CPU_CPU_BUS_CLK_HZ			CPU_CPU_MCLK_CLK_HZ
#define CPU_CPU_PER_CLK_HZ			CPU_CPU_MCLK_CLK_HZ
#define CPU_CPU_PITCLK_HZ			(CPU_CPU_MCLK_CLK_HZ / 16)		//Clock des Periodic Intervall Timer
#define CPU_CPU_SLOW_CLK_HZ			32768

#define CPU_BUS_CYCLE_TIME			(1000000000L / CPU_CPU_BUS_CLK_HZ) //in ns
#define CPU_SIO_CLOCK_HZ			CPU_CPU_PER_CLK_HZ

//IRQ Prioritten
#define CPU_IRQ_PRIOR_UNKNOWN		(-1)

#define CPU_PORT_MODE_CFG_IO_SHIFT			0x0
#define CPU_PORT_MODE_CFG_SLOPE_SHIFT		0x1
#define CPU_PORT_MODE_CFG_POSNEG_SHIFT		0x2
#define CPU_PORT_MODE_CFG_DUALSLOPE_SHIFT	0x3
#define CPU_PORT_MODE_CFG_PULL_SHIFT		0x4
#define CPU_PORT_MODE_CFG_PULLDIR_SHIFT		0x5
#define CPU_PORT_MODE_CFG_PER_SHIFT			0x6

#define CPU_PORT_MODE_OUTPUT				0x1		//b0000 0001

#define CPU_PORT_MODE_INPUT_LOW_ACT			0x0		//b0000 0000
#define CPU_PORT_MODE_INPUT_HIGH_ACT		0x4		//b0000 0100
#define CPU_PORT_MODE_INPUT_FALL_EDGE		0x2		//b0000 0010
#define CPU_PORT_MODE_INPUT_RIS_EDGE		0x6		//b0000 0110
#define CPU_PORT_MODE_INPUT_BOTH_EDGE		0x8		//b0000 1xx0

#define CPU_PORT_MODE_INPUT_PULLDOWN		0x10	//b0001 0000
#define CPU_PORT_MODE_INPUT_NOPULL			0x20	//b0010 0000
#define CPU_PORT_MODE_INPUT_PULLUP			0x30	//b0011 0000

#define CPU_PORT_MODE_INPUT					CPU_PORT_MODE_INPUT_NOPULL

#define CPU_PIN_OUT_LOW						0x0
#define CPU_PIN_OUT_HIGH					0x1
#define CPU_PIN_OUT_TOGGLE					0x2

#define CPU_PIN_PER_A						0x41	//0100 0001
#define CPU_PIN_PER_B						0x42	//0100 0010
#define CPU_PIN_PER_C						0x43	//0100 0011
#define CPU_PIN_PER_D						0x44	//0100 0100

enum cpu_port_list
{
	CPU_PORTA,
	CPU_PORTB,
	CPU_PORTC,
	CPU_PORTD,
	CPU_PORTE,
	CPU_PORT_NO,
};

enum cpu_irq_prior_list
{
	CPU_IRQ_PRIOR_SYS, 	//Systemmode Prioritt
	CPU_IRQ_PRIOR_IRQ_0,//Prioritten 0-7 fr IRQ
	CPU_IRQ_PRIOR_IRQ_1,
	CPU_IRQ_PRIOR_IRQ_2,
	CPU_IRQ_PRIOR_IRQ_3,
	CPU_IRQ_PRIOR_IRQ_4,
	CPU_IRQ_PRIOR_IRQ_5,
	CPU_IRQ_PRIOR_IRQ_6,
	CPU_IRQ_PRIOR_IRQ_7,
	CPU_IRQ_PRIOR_FIQ,	//FIQ - Prioritt
	CPU_IRQ_PRIOR_NO,	//Anzahl
};


//Liste der mglichen externen Interrupts (jedes Pin mglich)
enum cpu_ext_irq_list_porta
{
	CPU_EXT_IRQ_PORTA_0	= 0,
	CPU_EXT_IRQ_PORTA_1,
	CPU_EXT_IRQ_PORTA_2,
	CPU_EXT_IRQ_PORTA_3,
	CPU_EXT_IRQ_PORTA_4,
	CPU_EXT_IRQ_PORTA_5,
	CPU_EXT_IRQ_PORTA_6,
	CPU_EXT_IRQ_PORTA_7,
	CPU_EXT_IRQ_PORTA_8,
	CPU_EXT_IRQ_PORTA_9,
	CPU_EXT_IRQ_PORTA_10,
	CPU_EXT_IRQ_PORTA_11,
	CPU_EXT_IRQ_PORTA_12,
	CPU_EXT_IRQ_PORTA_13,
	CPU_EXT_IRQ_PORTA_14,
	CPU_EXT_IRQ_PORTA_15,
	CPU_EXT_IRQ_PORTA_16,
	CPU_EXT_IRQ_PORTA_17,
	CPU_EXT_IRQ_PORTA_18,
	CPU_EXT_IRQ_PORTA_19,
	CPU_EXT_IRQ_PORTA_20,
	CPU_EXT_IRQ_PORTA_21,
	CPU_EXT_IRQ_PORTA_22,
	CPU_EXT_IRQ_PORTA_23,
	CPU_EXT_IRQ_PORTA_24,
	CPU_EXT_IRQ_PORTA_25,
	CPU_EXT_IRQ_PORTA_26,
	CPU_EXT_IRQ_PORTA_27,
	CPU_EXT_IRQ_PORTA_28,
	CPU_EXT_IRQ_PORTA_29,
	CPU_EXT_IRQ_PORTA_30,
	CPU_EXT_IRQ_PORTA_31,
	CPU_EXT_IRQ_PORTA_NO
};
enum cpu_ext_irq_list_portb
{
	CPU_EXT_IRQ_PORTB_0	= CPU_EXT_IRQ_PORTA_NO,
	CPU_EXT_IRQ_PORTB_1,
	CPU_EXT_IRQ_PORTB_2,
	CPU_EXT_IRQ_PORTB_3,
	CPU_EXT_IRQ_PORTB_4,
	CPU_EXT_IRQ_PORTB_5,
	CPU_EXT_IRQ_PORTB_6,
	CPU_EXT_IRQ_PORTB_7,
	CPU_EXT_IRQ_PORTB_8,
	CPU_EXT_IRQ_PORTB_9,
	CPU_EXT_IRQ_PORTB_10,
	CPU_EXT_IRQ_PORTB_11,
	CPU_EXT_IRQ_PORTB_12,
	CPU_EXT_IRQ_PORTB_13,
	CPU_EXT_IRQ_PORTB_14,
	CPU_EXT_IRQ_PORTB_15,
	CPU_EXT_IRQ_PORTB_16,
	CPU_EXT_IRQ_PORTB_17,
	CPU_EXT_IRQ_PORTB_18,
	CPU_EXT_IRQ_PORTB_19,
	CPU_EXT_IRQ_PORTB_20,
	CPU_EXT_IRQ_PORTB_21,
	CPU_EXT_IRQ_PORTB_22,
	CPU_EXT_IRQ_PORTB_23,
	CPU_EXT_IRQ_PORTB_24,
	CPU_EXT_IRQ_PORTB_25,
	CPU_EXT_IRQ_PORTB_26,
	CPU_EXT_IRQ_PORTB_27,
	CPU_EXT_IRQ_PORTB_28,
	CPU_EXT_IRQ_PORTB_29,
	CPU_EXT_IRQ_PORTB_30,
	CPU_EXT_IRQ_PORTB_31,
	CPU_EXT_IRQ_PORTB_NO
};
enum cpu_ext_irq_list_portC
{
	CPU_EXT_IRQ_PORTC_0	= CPU_EXT_IRQ_PORTB_NO,
	CPU_EXT_IRQ_PORTC_1,
	CPU_EXT_IRQ_PORTC_2,
	CPU_EXT_IRQ_PORTC_3,
	CPU_EXT_IRQ_PORTC_4,
	CPU_EXT_IRQ_PORTC_5,
	CPU_EXT_IRQ_PORTC_6,
	CPU_EXT_IRQ_PORTC_7,
	CPU_EXT_IRQ_PORTC_8,
	CPU_EXT_IRQ_PORTC_9,
	CPU_EXT_IRQ_PORTC_10,
	CPU_EXT_IRQ_PORTC_11,
	CPU_EXT_IRQ_PORTC_12,
	CPU_EXT_IRQ_PORTC_13,
	CPU_EXT_IRQ_PORTC_14,
	CPU_EXT_IRQ_PORTC_15,
	CPU_EXT_IRQ_PORTC_16,
	CPU_EXT_IRQ_PORTC_17,
	CPU_EXT_IRQ_PORTC_18,
	CPU_EXT_IRQ_PORTC_19,
	CPU_EXT_IRQ_PORTC_20,
	CPU_EXT_IRQ_PORTC_21,
	CPU_EXT_IRQ_PORTC_22,
	CPU_EXT_IRQ_PORTC_23,
	CPU_EXT_IRQ_PORTC_24,
	CPU_EXT_IRQ_PORTC_25,
	CPU_EXT_IRQ_PORTC_26,
	CPU_EXT_IRQ_PORTC_27,
	CPU_EXT_IRQ_PORTC_28,
	CPU_EXT_IRQ_PORTC_29,
	CPU_EXT_IRQ_PORTC_30,
	CPU_EXT_IRQ_PORTC_31,
	CPU_EXT_IRQ_PORTC_NO
};
enum cpu_ext_irq_list_portd
{
	CPU_EXT_IRQ_PORTD_0	= CPU_EXT_IRQ_PORTC_NO,
	CPU_EXT_IRQ_PORTD_1,
	CPU_EXT_IRQ_PORTD_2,
	CPU_EXT_IRQ_PORTD_3,
	CPU_EXT_IRQ_PORTD_4,
	CPU_EXT_IRQ_PORTD_5,
	CPU_EXT_IRQ_PORTD_6,
	CPU_EXT_IRQ_PORTD_7,
	CPU_EXT_IRQ_PORTD_8,
	CPU_EXT_IRQ_PORTD_9,
	CPU_EXT_IRQ_PORTD_10,
	CPU_EXT_IRQ_PORTD_11,
	CPU_EXT_IRQ_PORTD_12,
	CPU_EXT_IRQ_PORTD_13,
	CPU_EXT_IRQ_PORTD_14,
	CPU_EXT_IRQ_PORTD_15,
	CPU_EXT_IRQ_PORTD_16,
	CPU_EXT_IRQ_PORTD_17,
	CPU_EXT_IRQ_PORTD_18,
	CPU_EXT_IRQ_PORTD_19,
	CPU_EXT_IRQ_PORTD_20,
	CPU_EXT_IRQ_PORTD_21,
	CPU_EXT_IRQ_PORTD_22,
	CPU_EXT_IRQ_PORTD_23,
	CPU_EXT_IRQ_PORTD_24,
	CPU_EXT_IRQ_PORTD_25,
	CPU_EXT_IRQ_PORTD_26,
	CPU_EXT_IRQ_PORTD_27,
	CPU_EXT_IRQ_PORTD_28,
	CPU_EXT_IRQ_PORTD_29,
	CPU_EXT_IRQ_PORTD_30,
	CPU_EXT_IRQ_PORTD_31,
	CPU_EXT_IRQ_PORTD_NO
};
enum cpu_ext_irq_list_porte
{
	CPU_EXT_IRQ_PORTE_0	= CPU_EXT_IRQ_PORTD_NO,
	CPU_EXT_IRQ_PORTE_1,
	CPU_EXT_IRQ_PORTE_2,
	CPU_EXT_IRQ_PORTE_3,
	CPU_EXT_IRQ_PORTE_4,
	CPU_EXT_IRQ_PORTE_5,
	CPU_EXT_IRQ_PORTE_6,
	CPU_EXT_IRQ_PORTE_7,
	CPU_EXT_IRQ_PORTE_8,
	CPU_EXT_IRQ_PORTE_9,
	CPU_EXT_IRQ_PORTE_10,
	CPU_EXT_IRQ_PORTE_11,
	CPU_EXT_IRQ_PORTE_12,
	CPU_EXT_IRQ_PORTE_13,
	CPU_EXT_IRQ_PORTE_14,
	CPU_EXT_IRQ_PORTE_15,
	CPU_EXT_IRQ_PORTE_16,
	CPU_EXT_IRQ_PORTE_17,
	CPU_EXT_IRQ_PORTE_18,
	CPU_EXT_IRQ_PORTE_19,
	CPU_EXT_IRQ_PORTE_20,
	CPU_EXT_IRQ_PORTE_21,
	CPU_EXT_IRQ_PORTE_22,
	CPU_EXT_IRQ_PORTE_23,
	CPU_EXT_IRQ_PORTE_24,
	CPU_EXT_IRQ_PORTE_25,
	CPU_EXT_IRQ_PORTE_26,
	CPU_EXT_IRQ_PORTE_27,
	CPU_EXT_IRQ_PORTE_28,
	CPU_EXT_IRQ_PORTE_29,
	CPU_EXT_IRQ_PORTE_30,
	CPU_EXT_IRQ_PORTE_31,
	CPU_EXT_IRQ_PORTE_NO,
	CPU_EXT_IRQ_NO			= CPU_EXT_IRQ_PORTE_NO
};

//Liste der Zeitgeber
enum cpu_timer_list
{
	CPU_TIMER_0		= 0,
	CPU_TIMER_1,
	CPU_TIMER_2,
#if CPU_TC1_AVAILABLE
	CPU_TIMER_3,
	CPU_TIMER_4,
	CPU_TIMER_5,
#endif
	CPU_TIMER_NO
};

enum cpu_adc_channel_list
{//Liste der CPU-internen A/D-Wandler
	CPU_ADC_CHANNEL_0,
	CPU_ADC_CHANNEL_1,
	CPU_ADC_CHANNEL_2,
	CPU_ADC_CHANNEL_3,
	CPU_ADC_CHANNEL_4,
	CPU_ADC_CHANNEL_5,
	CPU_ADC_CHANNEL_6,
	CPU_ADC_CHANNEL_7,
	CPU_ADC_CHANNEL_8,
	CPU_ADC_CHANNEL_9,
	CPU_ADC_CHANNEL_10,
	CPU_ADC_CHANNEL_11,
	CPU_ADC_CHANNEL_NO,
};

enum cpu_pid_list
{
	PID_FIQ,
	PID_SYS,
	PID_DBGU,
	PID_PIT,
	PID_WDT,
	PID_HSMC,
	PID_PIOA,
	PID_PIOB,
	PID_PIOC,
	PID_PIOD,
	PID_PIOE,
	PID_SMD,
	PID_USART0,
	PID_USART1,
	PID_USART2,
	PID_USART3,
	PID_UART0,
	PID_UART1,
	PID_TWI0,
	PID_TWI1,
	PID_TWI2,
	PID_HSMCI0,
	PID_HSMCI1,
	PID_HSMCI2,
	PID_SPI0,
	PID_SPI1,
	PID_TC0,
	PID_TC1,
	PID_PWM,
	PID_ADC,
	PID_DMAC0,
	PID_DMAC1,
	PID_UHPHS,
	PID_UDPHS,
	PID_GMAC,
	PID_EMAC,
	PID_LCDC,
	PID_ISI,
	PID_SSC0,
	PID_SSC1,
	PID_CAN0,
	PID_CAN1,
	PID_SHA,
	PID_AES,
	PID_TDES,
	PID_TRNG,
	PID_ARM,
	PID_IRQ,
	PID_FUSE,
	PID_MPDDRC,
	PID_NO,
};


enum cpu_irq_idx_list
{
	CPU_IRQ_IDX_FIQ,
	CPU_IRQ_IDX_SYS,
	CPU_IRQ_IDX_DBGU,
	CPU_IRQ_IDX_PIT,
	CPU_IRQ_IDX_WDT,
	CPU_IRQ_IDX_HSMC,
	CPU_IRQ_IDX_PIOA,
	CPU_IRQ_IDX_PIOB,
	CPU_IRQ_IDX_PIOC,
	CPU_IRQ_IDX_PIOD,
	CPU_IRQ_IDX_PIOE,
	CPU_IRQ_IDX_SMD,
	CPU_IRQ_IDX_USART0,
	CPU_IRQ_IDX_USART1,
	CPU_IRQ_IDX_USART2,
	CPU_IRQ_IDX_USART3,
	CPU_IRQ_IDX_UART0,
	CPU_IRQ_IDX_UART1,
	CPU_IRQ_IDX_TWI0,
	CPU_IRQ_IDX_TWI1,
	CPU_IRQ_IDX_TWI2,
	CPU_IRQ_IDX_HSMCI0,
	CPU_IRQ_IDX_HSMCI1,
	CPU_IRQ_IDX_HSMCI2,
	CPU_IRQ_IDX_SPI0,
	CPU_IRQ_IDX_SPI1,
	CPU_IRQ_IDX_TC0,
	CPU_IRQ_IDX_TC1,
	CPU_IRQ_IDX_PWM,
	CPU_IRQ_IDX_ADC,
	CPU_IRQ_IDX_DMAC0,
	CPU_IRQ_IDX_DMAC1,
	CPU_IRQ_IDX_UHPHS,
	CPU_IRQ_IDX_UDPHS,
	CPU_IRQ_IDX_GMAC,
	CPU_IRQ_IDX_EMAC,
	CPU_IRQ_IDX_LCDC,
	CPU_IRQ_IDX_ISI,
	CPU_IRQ_IDX_SSC0,
	CPU_IRQ_IDX_SSC1,
	CPU_IRQ_IDX_CAN0,
	CPU_IRQ_IDX_CAN1,
	CPU_IRQ_IDX_SHA,
	CPU_IRQ_IDX_AES,
	CPU_IRQ_IDX_TDES,
	CPU_IRQ_IDX_TRNG,
	CPU_IRQ_IDX_ARM,
	CPU_IRQ_IDX_IRQ,
	CPU_IRQ_IDX_FUSE,
	CPU_IRQ_IDX_MPDDRC,
	CPU_IRQ_IDX_SW0,
	CPU_IRQ_IDX_SW1,
	CPU_IRQ_IDX_SW2,
	CPU_IRQ_IDX_SW3,
	CPU_IRQ_IDX_SW4,
	CPU_IRQ_IDX_SW5,
	CPU_IRQ_IDX_SW6,
	CPU_IRQ_IDX_SW7,
	CPU_IRQ_IDX_NO,

	CPU_IRQ_IDX_EXT0	= CPU_IRQ_IDX_HSMCI0,
	CPU_IRQ_IDX_EXT1	= CPU_IRQ_IDX_HSMCI1,
	CPU_IRQ_IDX_EXT2	= CPU_IRQ_IDX_HSMCI2,
	CPU_IRQ_IDX_EXT3	= CPU_IRQ_IDX_GMAC,
	CPU_IRQ_IDX_EXT4	= CPU_IRQ_IDX_SSC0,
	CPU_IRQ_IDX_EXT5	= CPU_IRQ_IDX_SSC1,
	CPU_IRQ_IDX_EXT6	= CPU_IRQ_IDX_CAN0,
	CPU_IRQ_IDX_EXT7	= CPU_IRQ_IDX_CAN1,

	CPU_IRQ_IDX_RTOS_YIELD	= CPU_IRQ_IDX_TC1,

	CPU_IRQ_IDX_TIMER0	= CPU_IRQ_IDX_USART0,
	CPU_IRQ_IDX_TIMER1	= CPU_IRQ_IDX_USART1,
	CPU_IRQ_IDX_TIMER2	= CPU_IRQ_IDX_USART2
};

#define CPU_IRQ_IDX_SOFTWARE(_VAL_)		((_VAL_ == CPU_IRQ_PRIOR_IRQ_1) ? CPU_IRQ_IDX_SW1 :	\
										 (_VAL_ == CPU_IRQ_PRIOR_IRQ_2) ? CPU_IRQ_IDX_SW2 :	\
										 (_VAL_ == CPU_IRQ_PRIOR_IRQ_3) ? CPU_IRQ_IDX_SW3 :	\
										 (_VAL_ == CPU_IRQ_PRIOR_IRQ_4) ? CPU_IRQ_IDX_SW4 :	\
										 (_VAL_ == CPU_IRQ_PRIOR_IRQ_5) ? CPU_IRQ_IDX_SW5 :	\
										 (_VAL_ == CPU_IRQ_PRIOR_IRQ_6) ? CPU_IRQ_IDX_SW6 :	\
										 (_VAL_ == CPU_IRQ_PRIOR_IRQ_7) ? CPU_IRQ_IDX_SW7 :	\
										 CPU_IRQ_IDX_SW0)

//Exception-Vectors
#define VECTOR_RESET		(* (volatile unsigned long *) (0x00000000))
#define VECTOR_UNDEF_INSTR	(* (volatile unsigned long *) (0x00000004))
#define VECTOR_SUPVIS_CALL	(* (volatile unsigned long *) (0x00000008))
#define VECTOR_PREF_ABORT	(* (volatile unsigned long *) (0x0000000c))
#define VECTOR_DATA_ABORT	(* (volatile unsigned long *) (0x00000010))
#define VECTOR_unknown		(* (volatile unsigned long *) (0x00000014))
#define VECTOR_IRQ			(* (volatile unsigned long *) (0x00000018))
#define VECTOR_FIQ			(* (volatile unsigned long *) (0x0000001c))

// CPSR-Bits
#define CPU_CPSR_MODE_MASK			0x1f
#define CPU_CPSR_MODE_SYS			0x1f
#define CPU_CPSR_MODE_FIQ			0x11

#define CPU_CPSR_F_BIT				0x40
#define CPU_CPSR_I_BIT				0x80

#endif //IO_SAMA5D3x_h

